//
//
// Copyright 2021 gRPC authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
//

#ifndef GRPCPP_EXT_ADMIN_SERVICES_H
#define GRPCPP_EXT_ADMIN_SERVICES_H

#include <grpcpp/server_builder.h>

namespace grpc {

// Registers admin services to the given ServerBuilder. This function will add
// admin services based on build time dependencies, for example, it only adds
// CSDS service if xDS is enabled in this binary.
void AddAdminServices(grpc::ServerBuilder* builder);

}  // namespace grpc

#endif  // GRPCPP_EXT_ADMIN_SERVICES_H
